#include "testlib.h"

#include <iostream>
#include <set>
#include <utility>
#include <vector>

using namespace std;

int n, m, k;
vector<int> u, v;
int max_deg = 0;

vector<vector<pair<int, int>>> read_answer(InStream &s) {
  vector<vector<pair<int, int>>> ans;
  int p = s.readInt();
  for (int i = 0; i < p; i++) {
    ans.emplace_back();
    int L = s.readInt();
    for (int j = 0; j < L; j++) {
       int t = s.readInt();
       int g = s.readInt();
       t--; g--;
       ans.back().push_back(make_pair(t, g));
    }
  }
  return ans;
}

void check(InStream &s) {
  vector<vector<pair<int, int>>> ans = read_answer(s);

  if (ans.size() != max_deg) {
    s.quitf(_wa, "answer length is %d while the optimal is %d", (int)ans.size(), max_deg);
  }

  vector<bool> vis(u.size());
  for (const auto &list : ans) {
    set<int> t, g;
    for (const auto &p : list) {
      if (p.first < 0 || p.first >= n) {
        s.quitf(_pe, "invalid teacher index %d", p.first + 1);
      }
      if (p.second < 0 || p.second >= m) {
        s.quitf(_pe, "invalid group index %d", p.second + 1);
      }
      if (t.find(p.first) != t.end()) {
        s.quitf(_wa, "teacher %d was used twice", p.first + 1);
      }
      if (g.find(p.second) != g.end()) {
        s.quitf(_wa, "group %d was used twice", p.second + 1);
      }
      t.insert(p.first);
      g.insert(p.second);
      bool found = false;
      for (int i = 0; i < k; i++) {
        if (u[i] == p.first && v[i] == p.second && !vis[i]) {
          found = true;
          vis[i] = true;
          break;
        }
      }
      if (!found) {
        s.quitf(_wa, "extra pair (%d, %d)", p.first + 1, p.second + 1);
      }
    }
  }

  for (int i = 0; i < u.size(); i++) {
    if (!vis[i]) {
        s.quitf(_wa, "unused pair (%d, %d)", u[i] + 1, v[i] + 1);
    }
  }
}

int main(int argc, char * argv[]) {
  registerTestlibCmd(argc, argv);

  n = inf.readInt();
  m = inf.readInt();
  k = inf.readInt();

  u.resize(k);
  v.resize(k);
  vector<int> deg_l(n), deg_r(m);

  for (int i = 0; i < k; i++) {
    u[i] = inf.readInt();
    v[i] = inf.readInt();
    u[i]--;
    v[i]--;
    deg_l[u[i]]++;
    deg_r[v[i]]++;
    max_deg = max(max_deg, deg_l[u[i]]);
    max_deg = max(max_deg, deg_r[v[i]]);
  }

  check(ans);
  check(ouf);

  quitf(_ok, "ok");
}
